/* 
 * 
 * Copyright (c) 2001 Csar Blecua Udas    All Rights Reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * CESAR BLECUA UDIAS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
 * OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 * 
 */



/* Headers */
#include "public.h"


GLboolean	glaf_IsBlendColorSupported(void)
	{
	/*It returns GL_TRUE if glaf_BlendColor is supported, or GL_FALSE otherwise.
	
	  DESCRIPTION of the Blend Color feature:
	  
	  	* First of all, check if it's supported, calling glaf_IsBlendColorSupported.

		* glaf_BlendColor is used exactly the same as glBlendColor and glBlendColorEXT.

		* For it to work, use the following blending factors with glBlendFunc (they match the OpenGL "GL_" factors): 
			GLAF_CONSTANT_COLOR, GLAF_ONE_MINUS_CONSTANT_COLOR, GLAF_CONSTANT_ALPHA, GLAF_ONE_MINUS_CONSTANT_ALPHA.

		* The current "Blend Color" can be read passing GLAF_BLEND_COLOR to glGet*.
		
	
	  IMPORTANT: If glaf_BlendColor is unsupported and you call it, the application is likely to crash.
	*/
	if(GLB_current->glBlendColorPtr==NULL)
		{
		return GL_FALSE;
		}
	return GL_TRUE;
	}
	
	
void	glaf_BlendColor(GLclampf red, GLclampf green, GLclampf blue, GLclampf alpha)
	{
	/*Use is described at glaf_IsBlendColorSupported.
	*/
	GLB_current->glBlendColorPtr(red,green,blue,alpha);
	}
	
	
GLboolean	glaf_IsBlendMinMaxSupported(void)
	{
	/*It returns GL_TRUE if the MinMax blending equations are supported, or GL_FALSE otherwise.
	
	  DESCRIPTION of the Blend MinMax feature:
	  
	  	* First of all, check if it's supported, calling glaf_IsBlendMinMaxSupported.

		* Then use the GLAF_MIN or GLAF_MAX equations with glaf_BlendEquation.

		* To reset the blend equation to the default one, pass GLAF_FUNC_ADD to glaf_BlendEquation.

		* The current "Blend Equation" can be read passing GLAF_BLEND_EQUATION to glGet*.
		
	
	  IMPORTANT: If neither Blend MinMax nor Blend Subtract are supported, calling glaf_BlendEquation may 
	  	     cause the application to crash.
	*/
	
	return GLB_current->IsBlendMinMaxSupported;
	}
	

GLboolean	glaf_IsBlendSubtractSupported(void)
	{
	/*It returns GL_TRUE if the subtract blending equations are supported, or GL_FALSE otherwise.
	
	  DESCRIPTION of the Blend Subtract feature:
	  
	  	* First of all, check if it's supported, calling glaf_IsBlendSubtractSupported.

		* Then use the GLAF_FUNC_SUBTRACT or GLAF_FUNC_REVERSE_SUBTRACT equations with glaf_BlendEquation.

		* To reset the blend equation to the default one, pass GLAF_FUNC_ADD to glaf_BlendEquation.

		* The current "Blend Equation" can be read passing GLAF_BLEND_EQUATION to glGet*.
		
	
	  IMPORTANT: If neither Blend MinMax nor Blend Subtract are supported, calling glaf_BlendEquation may 
	  	     cause the application to crash.
	*/
	
	return GLB_current->IsBlendSubtractSupported;
	}
	

void	glaf_BlendEquation( GLenum mode )
	{
	/*Use is described at glaf_IsBlendMinMaxSupported and glaf_IsBlendSubtractSupported.
	*/
	GLB_current->glBlendEquationPtr(mode);
	}
	
	
GLboolean	glaf_IsRGBALogicOpSupported(void)
	{
	/*It returns GL_TRUE if LogicOp on RGBA is supported, or GL_FALSE otherwise.
	
	  DESCRIPTION of the LogicOp Blend feature:
	  
	  	* First of all, check if it's supported, calling glaf_IsRGBALogicOpSupported.
	
		* The Logic Op operation can be changed at *any* time with glLogicOp.
		
		* Call glaf_EnableRGBALogicOp to enable the feature.
		
		* Call glaf_DisableRGBALogicOp to disable it.
	
		* At this time there's no GLAF-supported way of querying whether the 
		  feature is enabled or disabled. If you need it, send email.
	
	
	  FUNCTIONALITY: The same of glEnable(GL_COLOR_LOGIC_OP) in OpenGL 1.1 and greater,
	  		 and the GL_EXT_blend_logic_op extension, which is available for all
			 versions if the implementation supports it.
	  
	  SIDE EFFECTS:
	  
	  	After calling glaf_DisableRGBALogicOp, blending will be disabled, and the
		blend equation will be GLAF_FUNC_ADD. These side effects will happen no
		matter the OpenGL version and supported extensions. If the OpenGL version 
		is 1.1 or greater, GL_COLOR_LOGIC_OP will be disabled as well.
		
		
	  EXPLANATION OF THE SIDE EFFECTS:
	  
	  	The only way for using logical operation for RGBA in OpenGL 1.0 is with
		the GL_EXT_blend_logic_op extension, it works through a blending
		equation (GL_LOGIC_OP), and drawing must be done with blending enabled.
		
		In OpenGL 1.1 and greater, logical operation for RGBA was added to the core,
		implemented in the same way as color index logicop (ie, with glEnable and
		glDisable instead of blending nor blend equation).
		
		Doing a clean preservation and management of the full OpenGL blending and
		logicop state for both 1.0 and 1.1 would complicate a bit the GLAF code,
		and so it has been decided to always disable blending and modify the
		blend equation after disabling the feature (note that this wouldn't be
		necessary for OpenGL 1.1 and greater, but it has been done for ensuring
		that side effects are the same for all implementations and for avoiding
		different behaviours of the same program).
	
	
	  IMPORTANT:
	  	
		If glaf_IsRGBALogicOpSupported returns NULL and you call either glaf_EnableRGBALogicOp
		or glaf_DisableRGBALogicOp, the application is likely to crash.
	*/
	
	if(GLB_current->_glaf_EnableRGBALogicOpPtr==NULL)
		{
		return GL_FALSE;
		}
	return GL_TRUE;
	}
	

void	glaf_EnableRGBALogicOp(void)
	{
	/*Use is described at glaf_IsRGBALogicOpSupported.
	*/
	GLB_current->_glaf_EnableRGBALogicOpPtr();
	}
	
		
void	glaf_DisableRGBALogicOp(void)
	{
	/*Use is described at glaf_IsRGBALogicOpSupported.
	*/
	GLB_current->_glaf_DisableRGBALogicOpPtr();
	}
	
		
	
	
