/* 
 * 
 * Copyright (c) 2001 Csar Blecua Udas    All Rights Reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * CESAR BLECUA UDIAS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
 * OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 * 
 */



/* Headers */

#ifdef _WIN32
#include <windows.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include "glincludes.h"
#include "glaf.h"
#include "contexts.h"

#include "abgrP.h"
#include "blending_extensionsP.h"
#include "color_subtableP.h"
#include "color_tableP.h"
#include "colormatrixP.h"
#include "convolutionP.h"
#include "convolution_border_modesP.h"
#include "copy_textureP.h"
#include "histogramP.h"
#include "packed_pixelsP.h"
#include "polygon_offsetP.h"
#include "subtextureP.h"
#include "texture_formatsP.h"
#include "texture_objectsP.h"
#include "separate_specular_colorP.h"
#include "texture_cube_mapP.h"


/***********************************/
/*Global pointer to current context*/
/***********************************/
struct GLAFST_context	*GLB_current;

	
/************************************************/
/*Local variables for OpenGL Contexts management*/
/************************************************/
static struct GLAFST_context	*ContextsArray=NULL;		/*array of contexts*/
static GLint			NumContexts=0;			/*num of contexts in array*/
void				(*_VerbosePtr)(char *)=NULL;	/*verbose callback (NULL by default)*/
	
void	_glaf_FillContextAllData(struct GLAFST_context *ctx);


/******************************************/
/*Functions for OpenGL Contexts management*/
/******************************************/
GLint	glaf_InitializeContext(void)
	{
	/* Allocates and initializes GLAF internal data for the current OpenGL context, and returns an unique ID which can be used
	   for restoring this context's data with glaf_ReadyForContext().
	   
	   Also calls glaf_ReadyForContext for the new context, so GLAF is prepared for beginning using the new context.
	   
	   This function *must* be called *once* for *each* context the user plans to use GLAF in. Each context must be created and 
	   made current *before* this function is called.
	*/
	GLint	f;
	
	if(NumContexts==0)
		{
		/*Initialize contexts array*/
		NumContexts++;
		ContextsArray=(struct GLAFST_context *)malloc(sizeof(struct GLAFST_context));
		}
	else
		{
		/*First we look for an unused context, in order to save memory*/
		for(f=0;f<NumContexts;f++)
			{
			if(ContextsArray[f].IsBeingUsed==GL_FALSE)
				{
				/*Unused context found*/
				_glaf_FillContextAllData(&ContextsArray[f]);
				glaf_ReadyForContext(f);
				return f;
				}
			}
		
		/*All contexts are in use. We need to enlarge the contexts array*/
		NumContexts++;
		ContextsArray=(struct GLAFST_context *)realloc(ContextsArray,NumContexts*sizeof(struct GLAFST_context));
		}
	_glaf_FillContextAllData(&ContextsArray[NumContexts-1]);
	glaf_ReadyForContext(NumContexts-1);
	return NumContexts-1;
	}
	
	
void	glaf_ReadyForContext(GLint id)
	{
	/* Makes GLAF ready for rendering into the context 'id'. The 'id' value should be the one that was returned when
	   glaf_InitializeContext() was used to initialize the context that you want GLAF to be ready for rendering in.
	   
	   Note that you don't need to use this function if your application has just one context, since glaf_InitializeContext() will
	   call this function for you when you initialize the context. However, using glaf_ReadyForContext() is mandatory for apps with
	   several contexts, and must be called after you make each context current (after the appropiate GLX, WGL or AGL MakeCurrent
	   call).
	
	   The reason for glaf_ReadyForContext is that OpenGL implementations can have different addresses for the same function in
	   different contexts (in other words, an OpenGL function can have a different address depending on the current context, and
	   so GLAF needs to be notified when the current context changes, since GLAF needs to keep track of pointers to extensions
	   functions).
	*/
	
	GLB_current= &ContextsArray[id];
	}
	

void	glaf_FreeContext(GLint id)
	{
	/* Free the context 'id'. The 'id' value should be the one that was returned when glaf_InitializeContext() was used to 
	   initialize the context that you want to free.
	   
	   After calling this function, the context 'id' is no longer usable, but note that the 'id' value can be reused 
	   by glaf_InitializeContext() if you create new contexts.
	
	   This function is necessary for applications that create and destroy contexts dynamically. On the other hand, applications
	   which create a fixed number of contexts at start-up don't need to use this function at all.
	*/
	GLint	f,last;
	
	/*Mark the context as unused*/
	ContextsArray[id].IsBeingUsed=GL_FALSE;
	
	/*Reallocate contexts array if necessary*/
	last=0;
	for(f=0;f<NumContexts;f++)
		{
		if(ContextsArray[f].IsBeingUsed==GL_FALSE)
			{
			last=f;
			}
		}
	last++;
	if(last<NumContexts)
		{
		NumContexts=last;
		ContextsArray=(struct GLAFST_context *)realloc(ContextsArray,NumContexts*sizeof(struct GLAFST_context));
		}
	}
	

/******************/
/*Verbose function*/
/******************/
void	glaf_SetVerbose(void funcptr(char *))
	{
	/*Sets a user-defined function for getting verbose messages when glaf_InitializeContext is called. GLAF generates verbose
	  message when calling glaf_InitializeContext only.
	  
	  If 'funcptr' is NULL, no verbose messages will be generated.
	  
	  Initially 'funcptr' is NULL, so if you want verbose messages, you need to call glaf_SetVerbose before glaf_InitializeContext.
	*/
	_VerbosePtr=funcptr;
	}

void	_glaf_Verbose(char *str)
	{
	if(_VerbosePtr!=NULL)
		{
		_VerbosePtr(str);
		}
	}
	

/***********************************/
/*Private functions (local to file)*/
/***********************************/
void	_glaf_FillContextAllData(struct GLAFST_context *ctx)
	{
	char		message[512];
	GLint		major,minor,release;
	
	/*Initialization verbose messages*/
	glaf_Version(&major,&minor,&release);
	sprintf(message,"GLAF %d.%d.%d",major,minor,release);
	_glaf_Verbose(message);
	
	glaf_OpenGLVersion(&major,&minor,&release);
	sprintf(message,"Initializing Context. OpenGL %d.%d.%d Detected.",major,minor,release);
	_glaf_Verbose(message);
	
	_glaf_Verbose("OpenGL Extensions String:");
	_glaf_Verbose((char *)glGetString(GL_EXTENSIONS));
	
	/*Mark context as used*/
	ctx->IsBeingUsed=GL_TRUE;
	
	/*Texture Formats*/
	_glaf_FillContextTextureFormats(ctx);
	
	/*Texture Objects*/
	_glaf_FillContextTextureObjects(ctx);
	
	/*Separate Specular Color*/
	_glaf_FillContextSeparateSpecularColor(ctx);
	
	/*SubTexture*/
	_glaf_FillContextSubTexture(ctx);
	
	/*Copy Texture*/
	_glaf_FillContextCopyTexture(ctx); /*IMPORTANT: never call before _glaf_FillContextSubTexture [see copy_textureP.c] */
	
	/*Blending Extensions*/
	_glaf_FillContextBlending(ctx);
	
	/*Polygon Offset*/
	_glaf_FillContextPolygonOffset(ctx);
	
	/*ABGR*/
	_glaf_FillContextABGR(ctx);
	
	/*Color Matrix*/
	_glaf_FillContextColorMatrix(ctx);
	
	/*Convolution*/
	_glaf_FillContextConvolution(ctx);
	
	/*Convolution Border Modes*/
	_glaf_FillContextConvolutionBorderModes(ctx);
	
	/*Histogram*/
	_glaf_FillContextHistogram(ctx);
	
	/*Color Table*/
	_glaf_FillContextColorTable(ctx);
	
	/*Color SubTable*/
	_glaf_FillContextColorSubTable(ctx);
	
	/*Packed Pixels*/
	_glaf_FillContextPackedPixels(ctx);
	
	/*Texture Cube Map*/
	_glaf_FillContextTextureCubeMap(ctx);
	
	return;
	}
	
