/* 
 * 
 * Copyright (c) 2001 Csar Blecua Udas    All Rights Reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * CESAR BLECUA UDIAS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
 * OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 * 
 */



/* Headers */
#include <stdlib.h>
#include "public.h"


static int	ForceEmulation=0; /*force emulation mode*/


GLboolean	glaf_IsCopyTextureNative(void)
	{
	/* It returns GL_TRUE if the copy texture feature is implemented in the system (either as GL_EXT_copy_texture or as a
	   core feature).
	   
	   It returns GL_FALSE if copy texture is emulated by GLAF because neither GL_EXT_copy_texture is available nor the core
	   supports it.
	
	   NOTE:
	   	-Please note that this function is named glaf_IsCopyTextureNative() rather than glaf_IsCopyTextureSupported().
		 There is no glaf_IsCopyTextureSupported() function, since copy texture is always supported (GLAF will emulate
		 it if the system doesn't support it).
	*/
	if(GLB_current->glCopyTexImage1DPtr==NULL)
		{
		/*Emulation mode*/
		return GL_FALSE;
		}
	/*Native support*/
	return GL_TRUE;
	}
	
	
GLboolean	glaf_IsCopySubTextureSupported(void)
	{
	/* It returns GL_TRUE if the "subtexture" part of the "copy texture" feature is supported. Note that the "copy texture"
	   feature is *always* supported, because it will be emulated by GLAF if the OpenGL implementation doesn't support it.
	   However, if neither "subtexture" nor "copy texture" are supported, GLAF won't be able to emulate the "subtexture" part 
	   of the "copy texture" feature.
	
	   If it returns GL_TRUE, the following functions can be used. If it returns GL_FALSE, they can't be used:
	   
	   					glaf_CopyTexSubImage1D
	   					glaf_CopyTexSubImage2D
	
	  MACHINE DEPENDENCIES: Some vendors partially support this feature in some of their implementations. For example, some older
	  			Silicon Graphics workstations have limitations on copy texture alignment and mipmap reloading.
				You should read your system-specific manpages or vendor documentation to find out if such limitations
				exist for your implementation.
	*/
	if(GLB_current->glCopyTexImage1DPtr==NULL)
		{
		/*Emulation mode*/
		if(!glaf_IsSubTextureSupported())
			{
			/* Copy SubTexture can't be emulated */
			return GL_FALSE;
			}
		}
	return GL_TRUE;
	}
	

void	glaf_CopyTextureForceEmulation(int mode)
	{
	/*Not really useful for applications. This function is meant for GLAF development, in order to test the emulation mode.
	
	  mode=0  -> Normal behaviour.
	  mode!=0 -> Force emulation mode.
	*/
	
	ForceEmulation=mode;
	}
	
	
void	glaf_CopyTexImage1D( GLenum target, GLint level, GLenum internalformat, GLint x, GLint y, GLsizei width, GLint border )
	{
	if((GLB_current->glCopyTexImage1DPtr==NULL)||(ForceEmulation))
		{
		/*Emulation mode*/
		GLenum 	pixeltype,baseformat;
		GLvoid	*buffer;
		
		/* Guess data type and color format */
		pixeltype=glaf_BestUnsignedPixelType(glaf_FrameBufferMaxColorSize());
		baseformat=glaf_BaseColorFormat(internalformat);
		if(baseformat==GLAF_INTENSITY)
			{
			/* GL_INTENSITY is not supported by glReadPixels, but we can use GL_LUMINANCE instead. 
			 * The final texture will be GL_INTENSITY, because we'll use the requested format for loading it. */
			baseformat=GL_LUMINANCE;
			}
		
		
		/*NOTE: We don't care about 1D Convolution, because it can only be applied to glTexImage1D (not glReadPixels),
		        and it won't be applied then either, because we disable everything for convolution (so, convolution is
			*not* supported for glaf_CopyTexImage1D in emulation mode).*/
		/*IMPORTANT: 2D Convolution can affect the glReadPixels operation, destroying the image since height=1. This is
		             a bug in this emulation. This bug doesn't affect glaf_CopyTexImage2D, just glaf_CopyTexImage1D */
		
		/* Allocate temp buffer */
		buffer=(GLvoid *)malloc(glaf_ImageSize(width,1,baseformat,pixeltype,4));
		
		/* Save and Reset the PixelStore state */
		glaf_StorePixelPackState();
		glaf_ResetPixelPackState();
		glaf_StorePixelUnpackState();
		glaf_ResetPixelUnpackState();
		
		/* ReadPixels into temp buffer. Note that any pixeltransfer and imaging operations *can* be enabled */
		glReadPixels(x,y,width,1,baseformat,pixeltype,buffer);
		
		/* Save and Reset the pixeltransfer and imaging state (disable *everything* for texture loading) */
		glaf_StorePixelReadState();
		glaf_ResetPixelReadState();
		
		/* Load texture, with all imaging/pixeltransfer operations disabled in order to avoid performing them twice */
		glTexImage1D(target,level,internalformat,width,border,baseformat,pixeltype,buffer);
		
		/* Restore the pixeltransfer and imaging state to their previous values */
		glaf_RestorePixelReadState();
		
		/* Restore the PixelStore state to its previous values */
		glaf_RestorePixelPackState();
		glaf_RestorePixelUnpackState();
		
		/* Free the temp buffer */
		free(buffer);
		}
	else
		{
		/*Normal mode*/
		GLB_current->glCopyTexImage1DPtr( target, level, internalformat, x, y, width, border );
		}
	}
	
	
void	glaf_CopyTexImage2D( GLenum target, GLint level, GLenum internalformat, GLint x, GLint y, GLsizei width, GLsizei height, GLint border )
	{
	if((GLB_current->glCopyTexImage1DPtr==NULL)||(ForceEmulation))
		{
		/*Emulation mode*/
		GLenum 	pixeltype,baseformat;
		GLvoid	*buffer;
		GLint	nwidth,nheight;
		
		/* Guess data type and color format */
		pixeltype=glaf_BestUnsignedPixelType(glaf_FrameBufferMaxColorSize());
		baseformat=glaf_BaseColorFormat(internalformat);
		if(baseformat==GLAF_INTENSITY)
			{
			/* GL_INTENSITY is not supported by glReadPixels, but we can use GL_LUMINANCE instead. 
			 * The final texture will be GL_INTENSITY, because we'll use the requested format for loading it. */
			baseformat=GL_LUMINANCE;
			}
		
		/* Check whether convolution (if enabled) resizes the original image */
		/* NOTE: 1D Convolution is *never* applied to 2D images */
		nwidth=width+glaf_ImageWidthConvolution2DIncrement();
		nheight=height+glaf_ImageHeightConvolution2DIncrement();
		if((nwidth<1)||(nheight<1))
			{
			/*The OpenGL spec says no image and no error are generated in this case*/
			return;
			}
		
		/* Allocate temp buffer */
		buffer=(GLvoid *)malloc(glaf_ImageSize(nwidth,nheight,baseformat,pixeltype,4));
		
		/* Save and Reset the PixelStore state */
		glaf_StorePixelPackState();
		glaf_ResetPixelPackState();
		glaf_StorePixelUnpackState();
		glaf_ResetPixelUnpackState();
		
		/* ReadPixels into temp buffer. Note that any pixeltransfer and imaging operations *can* be enabled */
		/* Note: GL_SGIX_interlace doesn't affect glReadPixels, but it will be applied later (if enabled) in glTexImage */
		glReadPixels(x,y,width,height,baseformat,pixeltype,buffer);
		
		/* Save and Reset the pixeltransfer and imaging state (disable *everything* for texture loading) */
		glaf_StorePixelReadState();
		glaf_ResetPixelReadState();
		
		/* Load texture, with all imaging/pixeltransfer operations disabled in order to avoid performing them twice */
		glTexImage2D(target,level,internalformat,nwidth,nheight,border,baseformat,pixeltype,buffer);
		
		/* Restore the pixeltransfer and imaging state to their previous values */
		glaf_RestorePixelReadState();
		
		/* Restore the PixelStore state to its previous values */
		glaf_RestorePixelPackState();
		glaf_RestorePixelUnpackState();
		
		/* Free the temp buffer */
		free(buffer);
		}
	else
		{
		/*Normal mode*/
		GLB_current->glCopyTexImage2DPtr( target, level, internalformat, x, y, width, height, border );
		}
	}
	
	
void	glaf_CopyTexSubImage1D( GLenum target, GLint level, GLint xoffset, GLint x, GLint y, GLsizei width )
	{
	if((GLB_current->glCopyTexImage1DPtr==NULL)||(ForceEmulation))
		{
		/*Emulation mode*/
		GLenum 	pixeltype,baseformat;
		GLint	internalformat;
		GLvoid	*buffer;
		
		if(!glaf_IsSubTextureSupported())
			{
			/* Copy SubTexture can't be emulated */
			return;
			}
		
		/* Guess data type and color format */
		pixeltype=glaf_BestUnsignedPixelType(glaf_FrameBufferMaxColorSize());
		glGetTexLevelParameteriv(GL_TEXTURE_1D, level, GL_TEXTURE_COMPONENTS, &internalformat );
		baseformat=glaf_BaseColorFormat((GLenum)internalformat);
		if(baseformat==GLAF_INTENSITY)
			{
			/* GL_INTENSITY is not supported by glReadPixels, but we can use GL_LUMINANCE instead. 
			 * The final texture will be GL_INTENSITY, because we'll use the requested format for loading it. */
			baseformat=GL_LUMINANCE;
			}
		
		
		/*NOTE: We don't care about 1D Convolution, because it can only be applied to glTexImage1D (not glReadPixels),
		        and it won't be applied then either, because we disable everything for convolution (so, convolution is
			*not* supported for glaf_CopyTexSubImage1D in emulation mode).*/
		/*IMPORTANT: 2D Convolution can affect the glReadPixels operation, destroying the image since height=1. This is
		             a bug in this emulation. */
		
		/* Allocate temp buffer */
		buffer=(GLvoid *)malloc(glaf_ImageSize(width,1,baseformat,pixeltype,4));
		
		/* Save and Reset the PixelStore state */
		glaf_StorePixelPackState();
		glaf_ResetPixelPackState();
		glaf_StorePixelUnpackState();
		glaf_ResetPixelUnpackState();
		
		/* ReadPixels into temp buffer. Note that any pixeltransfer and imaging operations *can* be enabled */
		glReadPixels(x,y,width,1,baseformat,pixeltype,buffer);
		
		/* Save and Reset the pixeltransfer and imaging state (disable *everything* for texture loading) */
		glaf_StorePixelReadState();
		glaf_ResetPixelReadState();
		
		/* Load texture, with all imaging/pixeltransfer operations disabled in order to avoid performing them twice */
		glaf_TexSubImage1D(target,level,xoffset,width,baseformat,pixeltype,buffer);
		
		/* Restore the pixeltransfer and imaging state to their previous values */
		glaf_RestorePixelReadState();
		
		/* Restore the PixelStore state to its previous values */
		glaf_RestorePixelPackState();
		glaf_RestorePixelUnpackState();
		
		/* Free the temp buffer */
		free(buffer);
		}
	else
		{
		/*Normal mode*/
		GLB_current->glCopyTexSubImage1DPtr( target, level, xoffset, x, y, width );
		}
	}
	
	
void	glaf_CopyTexSubImage2D( GLenum target, GLint level, GLint xoffset, GLint yoffset, GLint x, GLint y, GLsizei width, GLsizei height )
	{
	if((GLB_current->glCopyTexImage1DPtr==NULL)||(ForceEmulation))
		{
		/*Emulation mode*/
		GLenum 	pixeltype,baseformat;
		GLint	internalformat;
		GLvoid	*buffer;
		GLint	nwidth,nheight;
		
		if(!glaf_IsSubTextureSupported())
			{
			/* Copy SubTexture can't be emulated */
			return;
			}
		
		/* Guess data type and color format */
		pixeltype=glaf_BestUnsignedPixelType(glaf_FrameBufferMaxColorSize());
		glGetTexLevelParameteriv(GL_TEXTURE_2D, level, GL_TEXTURE_COMPONENTS, &internalformat );
		baseformat=glaf_BaseColorFormat((GLenum)internalformat);
		if(baseformat==GLAF_INTENSITY)
			{
			/* GL_INTENSITY is not supported by glReadPixels, but we can use GL_LUMINANCE instead. 
			 * The final texture will be GL_INTENSITY, because we'll use the requested format for loading it. */
			baseformat=GL_LUMINANCE;
			}
		
		/* Check whether convolution (if enabled) resizes the original image */
		/* NOTE: 1D Convolution is *never* applied to 2D images */
		nwidth=width+glaf_ImageWidthConvolution2DIncrement();
		nheight=height+glaf_ImageHeightConvolution2DIncrement();
		if((nwidth<1)||(nheight<1))
			{
			/*The OpenGL spec says no image and no error are generated in this case*/
			return;
			}
		
		/* Allocate temp buffer */
		buffer=(GLvoid *)malloc(glaf_ImageSize(nwidth,nheight,baseformat,pixeltype,4));
		
		/* Save and Reset the PixelStore state */
		glaf_StorePixelPackState();
		glaf_ResetPixelPackState();
		glaf_StorePixelUnpackState();
		glaf_ResetPixelUnpackState();
		
		/* ReadPixels into temp buffer. Note that any pixeltransfer and imaging operations *can* be enabled */
		/* Note: GL_SGIX_interlace doesn't affect glReadPixels, but it will be applied later (if enabled) in glTexImage */
		glReadPixels(x,y,width,height,baseformat,pixeltype,buffer);
		
		/* Save and Reset the pixeltransfer and imaging state (disable *everything* for texture loading) */
		glaf_StorePixelReadState();
		glaf_ResetPixelReadState();
		
		/* Load subtexture, with all imaging/pixeltransfer operations disabled in order to avoid performing them twice */
		glaf_TexSubImage2D(target,level,xoffset,yoffset,nwidth,nheight,baseformat,pixeltype,buffer);
		
		/* Restore the pixeltransfer and imaging state to their previous values */
		glaf_RestorePixelReadState();
		
		/* Restore the PixelStore state to its previous values */
		glaf_RestorePixelPackState();
		glaf_RestorePixelUnpackState();
		
		/* Free the temp buffer */
		free(buffer);
		}
	else
		{
		/*Normal mode*/
		GLB_current->glCopyTexSubImage2DPtr( target, level, xoffset, yoffset, x, y, width, height );
		}
	}
	
	

