/* 
 * 
 * Copyright (c) 2001 Csar Blecua Udas    All Rights Reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * CESAR BLECUA UDIAS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
 * OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 * 
 */



/* Headers */

#ifdef _WIN32
#include <windows.h>
#endif

#include "glincludes.h"
#include "glaf.h"


GLint	glaf_ImageSize(GLint x, GLint y, GLenum format, GLenum type, GLint align)
	{
	/* Returns the size (in bytes) of a image of 'x' by 'y' pixels, given a color format, data type, and glPixelStore alignment.
	
	   If any of the arguments are unsupported or absurd, zero is returned.
	
	   CURRENT LIMITATIONS: Neither packed pixels data types, nor sized texture formats are correctly managed by this function.
	   			So, the current version will give an incorrect result for such arguments.
	*/
	GLint		components,bytes,rawscan,r;
	
	if((x<=0)||(y<=0))
		{
		return 0;
		}
	
	bytes=glaf_TypeSize(type);
	if(bytes==0)
		{
		return 0;
		}
		
	components=glaf_NumComponentsFormat(format);
	if(components==0)
		{
		return 0;
		}
	
	rawscan=components*bytes*x;
	r=rawscan%align;
	if(r==0)
		{
		return rawscan*y;
		}
	else
		{
		return (rawscan-r+align)*y;
		}
	}
	

GLint	glaf_FrameBufferMaxColorSize(void)
	{
	/* Returns the number of bytes necessary for storing each color component of one pixel of the CURRENT framebuffer if all color
	   components have the same size, or the number of bytes necessary for storing the LARGEST color component of one pixel
	   if not all components have the same size.
	
	   The smallest returned value is 1 byte. The function assumes different color components won't share the same byte (ie, the
	   packed pixels formats are not taken into account).
	
	   Only color components are considered (R, G, B, A). So, Stencil, Depth, and Accumulation fall outside the scope of this 
	   function.
	
	   Examples:
	   		15bit (  5 +  5 +  5 +  0 ) framebuffers -> returns 1
			16bit (  5 +  6 +  5 +  0 ) framebuffers -> returns 1
			16bit (  5 +  5 +  5 +  1 ) framebuffers -> returns 1
			24bit (  8 +  8 +  8 +  0 ) framebuffers -> returns 1
			32bit (  8 +  8 +  8 +  8 ) framebuffers -> returns 1
			32bit ( 10 + 10 + 10 +  2 ) framebuffers -> returns 2
			36bit ( 12 + 12 + 12 +  0 ) framebuffers -> returns 2
			48bit ( 12 + 12 + 12 + 12 ) framebuffers -> returns 2
			
	*/
	GLint	r, g, b, a, bits;
	
	glGetIntegerv(GL_RED_BITS, &r);
	glGetIntegerv(GL_GREEN_BITS, &g);
	glGetIntegerv(GL_BLUE_BITS, &b);
	glGetIntegerv(GL_ALPHA_BITS, &a);
	
	bits=8;
	if(r>bits)
		{
		bits=r;
		}
	if(g>bits)
		{
		bits=g;
		}
	if(b>bits)
		{
		bits=b;
		}
	if(a>bits)
		{
		bits=a;
		}
	if((bits%8)==0)
		{
		/* No bits are wasted */
		return bits/8;
		}
	/* Some bits are wasted */
	return (bits/8)+1;
	}
	
	
GLenum	glaf_BestUnsignedPixelType(GLint size)
	{
	/* Returns the smallest OpenGL unsigned pixel data type that can hold 'size' bytes.
	
	   Examples:
	   		For size=1, GL_UNSIGNED_BYTE
			For size=2, GL_UNSIGNED_SHORT
			For size=3, GL_UNSIGNED_INT
			For size=4, GL_UNSIGNED_INT
			For size>4, GL_FALSE (error, OpenGL doesn't support such size).
	*/
	switch(size)
		{
		case 1:
			return GL_UNSIGNED_BYTE;
		case 2:
			return GL_UNSIGNED_SHORT;
		case 3:
		case 4:
			return GL_UNSIGNED_INT;
		default:
			return GL_FALSE;
		}
	}
	
	
GLenum	glaf_BestSignedPixelType(GLint size)
	{
	/* Returns the smallest OpenGL signed pixel data type that can hold 'size' bytes.
	
	   Examples:
	   		For size=1, GL_BYTE
			For size=2, GL_SHORT
			For size=3, GL_INT
			For size=4, GL_INT
			For size>4, GL_FALSE (error, OpenGL doesn't support such size).
	*/
	switch(size)
		{
		case 1:
			return GL_BYTE;
		case 2:
			return GL_SHORT;
		case 3:
		case 4:
			return GL_INT;
		default:
			return GL_FALSE;
		}
	}
	

GLenum	glaf_BaseColorFormat(GLenum format)
	{
	/* Returns the "base" color format for a given color format. Non-sized formats are left unchanged (so, GL_RGBA returns GL_RGBA,
	   GLAF_ABGR returns GLAF_ABGR, and so on). Sized formats return their unsized version (so, GLAF_RGB8 returns GL_RGB).
	
	   If 'format' is unsupported, the same value is returned unchanged.
	   
	   Possible returned formats:
				GL_RED
				GL_GREEN
				GL_BLUE
				GL_ALPHA
				GL_LUMINANCE
				GLAF_INTENSITY	
				GL_COLOR_INDEX
				GL_STENCIL_INDEX
				GL_DEPTH_COMPONENT
				GL_LUMINANCE_ALPHA
				GL_RGB
				GL_RGBA
				GLAF_ABGR
	
	   Supported formats:
				GL_RED
				GL_GREEN
				GL_BLUE
				GL_ALPHA
				GLAF_ALPHA4	
				GLAF_ALPHA8	
				GLAF_ALPHA12	
				GLAF_ALPHA16	
				GL_LUMINANCE
				GLAF_LUMINANCE4	
				GLAF_LUMINANCE8	
				GLAF_LUMINANCE12
				GLAF_LUMINANCE16
				GLAF_INTENSITY	
				GLAF_INTENSITY4	
				GLAF_INTENSITY8	
				GLAF_INTENSITY12
				GLAF_INTENSITY16
				GL_COLOR_INDEX
				GL_STENCIL_INDEX
				GL_DEPTH_COMPONENT
				GL_LUMINANCE_ALPHA
				GLAF_LUMINANCE4_ALPHA4	
				GLAF_LUMINANCE6_ALPHA2	
				GLAF_LUMINANCE8_ALPHA8	
				GLAF_LUMINANCE12_ALPHA4	
				GLAF_LUMINANCE12_ALPHA12
				GLAF_LUMINANCE16_ALPHA16
				GL_RGB
				GLAF_RGB4
				GLAF_RGB5
				GLAF_RGB8
				GLAF_RGB10
				GLAF_RGB12
				GLAF_RGB16
				GL_RGBA
				GLAF_RGBA2
				GLAF_RGBA4
				GLAF_RGB5_A1
				GLAF_RGBA8
				GLAF_RGB10_A2
				GLAF_RGBA12
				GLAF_RGBA16
				GLAF_ABGR
	*/
	switch(format)
		{
		case GLAF_ALPHA4:
		case GLAF_ALPHA8:
		case GLAF_ALPHA12:
		case GLAF_ALPHA16:
	
			return GL_ALPHA;
	
		case GLAF_LUMINANCE4:
		case GLAF_LUMINANCE8:
		case GLAF_LUMINANCE12:
		case GLAF_LUMINANCE16:
	
			return GL_LUMINANCE;
	
		case GLAF_INTENSITY4:
		case GLAF_INTENSITY8:
		case GLAF_INTENSITY12:
		case GLAF_INTENSITY16:
	
			return GLAF_INTENSITY;
	
		case GLAF_LUMINANCE4_ALPHA4:
		case GLAF_LUMINANCE6_ALPHA2:
		case GLAF_LUMINANCE8_ALPHA8:
		case GLAF_LUMINANCE12_ALPHA4:
		case GLAF_LUMINANCE12_ALPHA12:
		case GLAF_LUMINANCE16_ALPHA16:
		
			return GL_LUMINANCE_ALPHA;
			
		case GLAF_RGB4:
		case GLAF_RGB5:
		case GLAF_RGB8:
		case GLAF_RGB10:
		case GLAF_RGB12:
		case GLAF_RGB16:
		
			return GL_RGB;
			
		case GLAF_RGBA2:
		case GLAF_RGBA4:
		case GLAF_RGB5_A1:
		case GLAF_RGBA8:
		case GLAF_RGB10_A2:
		case GLAF_RGBA12:
		case GLAF_RGBA16:
		
			return GL_RGBA;
			
		default:

			return format;
		}
	}
	
		
GLint	glaf_NumComponentsFormat(GLenum format)
	{
	/* Returns the number of components of a given OpenGL color format.
	
	   If the format is unknown/unsupported, zero is returned.
	
	   Supported formats:
				GL_RED
				GL_GREEN
				GL_BLUE
				GL_ALPHA
				GLAF_ALPHA4	
				GLAF_ALPHA8	
				GLAF_ALPHA12	
				GLAF_ALPHA16	
				GL_LUMINANCE
				GLAF_LUMINANCE4	
				GLAF_LUMINANCE8	
				GLAF_LUMINANCE12
				GLAF_LUMINANCE16
				GLAF_INTENSITY	
				GLAF_INTENSITY4	
				GLAF_INTENSITY8	
				GLAF_INTENSITY12
				GLAF_INTENSITY16
				GL_COLOR_INDEX
				GL_STENCIL_INDEX
				GL_DEPTH_COMPONENT
				GL_LUMINANCE_ALPHA
				GLAF_LUMINANCE4_ALPHA4	
				GLAF_LUMINANCE6_ALPHA2	
				GLAF_LUMINANCE8_ALPHA8	
				GLAF_LUMINANCE12_ALPHA4	
				GLAF_LUMINANCE12_ALPHA12
				GLAF_LUMINANCE16_ALPHA16
				GL_RGB
				GLAF_RGB4
				GLAF_RGB5
				GLAF_RGB8
				GLAF_RGB10
				GLAF_RGB12
				GLAF_RGB16
				GL_RGBA
				GLAF_RGBA2
				GLAF_RGBA4
				GLAF_RGB5_A1
				GLAF_RGBA8
				GLAF_RGB10_A2
				GLAF_RGBA12
				GLAF_RGBA16
				GLAF_ABGR
	   	
	*/
	switch(format)
		{
		case GL_RED:
		case GL_GREEN:
		case GL_BLUE:
		case GL_ALPHA:
		case GLAF_ALPHA4:
		case GLAF_ALPHA8:
		case GLAF_ALPHA12:
		case GLAF_ALPHA16:
		case GL_LUMINANCE:
		case GLAF_LUMINANCE4:
		case GLAF_LUMINANCE8:
		case GLAF_LUMINANCE12:
		case GLAF_LUMINANCE16:
		case GLAF_INTENSITY:
		case GLAF_INTENSITY4:
		case GLAF_INTENSITY8:
		case GLAF_INTENSITY12:
		case GLAF_INTENSITY16:
		case GL_COLOR_INDEX:
		case GL_STENCIL_INDEX:
		case GL_DEPTH_COMPONENT:
		
			return 1;
			
		case GL_LUMINANCE_ALPHA:
		case GLAF_LUMINANCE4_ALPHA4:
		case GLAF_LUMINANCE6_ALPHA2:
		case GLAF_LUMINANCE8_ALPHA8:
		case GLAF_LUMINANCE12_ALPHA4:
		case GLAF_LUMINANCE12_ALPHA12:
		case GLAF_LUMINANCE16_ALPHA16:
		
			return 2;
			
		case GL_RGB:
		case GLAF_RGB4:
		case GLAF_RGB5:
		case GLAF_RGB8:
		case GLAF_RGB10:
		case GLAF_RGB12:
		case GLAF_RGB16:
		
			return 3;
			
		case GL_RGBA:
		case GLAF_RGBA2:
		case GLAF_RGBA4:
		case GLAF_RGB5_A1:
		case GLAF_RGBA8:
		case GLAF_RGB10_A2:
		case GLAF_RGBA12:
		case GLAF_RGBA16:
		case GLAF_ABGR:

			return 4;
			
		default:

			return 0;
		}
	}
	

GLint	glaf_TypeSize(GLenum type)
	{
	/* Returns the size (in bytes) of a given OpenGL data type, or zero if the type is unknown/unsupported.
	
	   Supported types:
				GL_UNSIGNED_BYTE
				GL_BYTE
				GLAF_UNSIGNED_BYTE_3_3_2
				GL_UNSIGNED_SHORT
				GL_SHORT
				GLAF_UNSIGNED_SHORT_4_4_4_4
				GLAF_UNSIGNED_SHORT_5_5_5_1
				GL_UNSIGNED_INT
				GL_INT
				GLAF_UNSIGNED_INT_8_8_8_8
				GLAF_UNSIGNED_INT_10_10_10_2
				GL_FLOAT
	
	   NOTE:   The GL_BITMAP type returns zero (this behaviour may be changed in future releases).
	*/
	switch(type)
		{
		case GL_UNSIGNED_BYTE:
		case GL_BYTE:
		case GLAF_UNSIGNED_BYTE_3_3_2:

			return 1;
		
		case GL_UNSIGNED_SHORT:
		case GL_SHORT:
		case GLAF_UNSIGNED_SHORT_4_4_4_4:
		case GLAF_UNSIGNED_SHORT_5_5_5_1:

			return 2;
		
		case GL_UNSIGNED_INT:
		case GL_INT:
		case GLAF_UNSIGNED_INT_8_8_8_8:
		case GLAF_UNSIGNED_INT_10_10_10_2:

			return 4;
		
		case GL_FLOAT:
		
			return(sizeof(float));
			
		case GL_BITMAP:
		default:
			
			return 0;
		}
	}


GLint	glaf_ImageWidthConvolution2DIncrement(void)
	{
	/* It returns the (signed) width increment of images after applying 2D convolution to them. It will always be either 0 or
	   a negative integer (images never grow after convolution, they can either shrink or keep the same size, depending on the
	   convolution border mode).
	
	   If convolution is not supported, returns 0.
	   
	   If both GLAF_CONVOLUTION_2D and GLAF_SEPARABLE_2D are disabled, it returns 0.
	   
	   If GLAF_CONVOLUTION_2D is enabled, its kernel is the one used for computing the increment.
	   
	   If GLAF_CONVOLUTION_2D is disabled and GLAF_SEPARABLE_2D is enabled, the GLAF_SEPARABLE_2D kernel is used instead.
	*/
	
	GLint	bordermode, kwidth;
	
	if(!glaf_IsConvolutionSupported())
		{
		/*Convolution not supported*/
		return 0;
		}
	if(glIsEnabled(GLAF_CONVOLUTION_2D))
		{
		if(glaf_IsConvolutionBorderModesSupported())
			{
			glaf_GetConvolutionParameteriv(GLAF_CONVOLUTION_2D, GLAF_CONVOLUTION_BORDER_MODE, &bordermode);
			if(bordermode!=GLAF_REDUCE)
				{
				/*Image keeps the same size after convolution*/
				return 0;
				}
			}
		glaf_GetConvolutionParameteriv(GLAF_CONVOLUTION_2D, GLAF_CONVOLUTION_WIDTH, &kwidth);
		return 1-kwidth;
		}
	if(glIsEnabled(GLAF_SEPARABLE_2D))
		{
		if(glaf_IsConvolutionBorderModesSupported())
			{
			glaf_GetConvolutionParameteriv(GLAF_SEPARABLE_2D, GLAF_CONVOLUTION_BORDER_MODE, &bordermode);
			if(bordermode!=GLAF_REDUCE)
				{
				/*Image keeps the same size after convolution*/
				return 0;
				}
			}
		glaf_GetConvolutionParameteriv(GLAF_SEPARABLE_2D, GLAF_CONVOLUTION_WIDTH, &kwidth);
		return 1-kwidth;
		}
	/*Convolution disabled*/
	return 0;
	}
	
	
GLint	glaf_ImageHeightConvolution2DIncrement(void)
	{
	/* It returns the (signed) height increment of images after applying 2D convolution to them. It will always be either 0 or
	   a negative integer (images never grow after convolution, they can either shrink or keep the same size, depending on the
	   convolution border mode).
	
	   If convolution is not supported, returns 0.
	   
	   If both GLAF_CONVOLUTION_2D and GLAF_SEPARABLE_2D are disabled, it returns 0.
	   
	   If GLAF_CONVOLUTION_2D is enabled, its kernel is the one used for computing the increment.
	   
	   If GLAF_CONVOLUTION_2D is disabled and GLAF_SEPARABLE_2D is enabled, the GLAF_SEPARABLE_2D kernel is used instead.
	*/
	
	GLint	bordermode, kheight;
	
	if(!glaf_IsConvolutionSupported())
		{
		/*Convolution not supported*/
		return 0;
		}
	if(glIsEnabled(GLAF_CONVOLUTION_2D))
		{
		if(glaf_IsConvolutionBorderModesSupported())
			{
			glaf_GetConvolutionParameteriv(GLAF_CONVOLUTION_2D, GLAF_CONVOLUTION_BORDER_MODE, &bordermode);
			if(bordermode!=GLAF_REDUCE)
				{
				/*Image keeps the same size after convolution*/
				return 0;
				}
			}
		glaf_GetConvolutionParameteriv(GLAF_CONVOLUTION_2D, GLAF_CONVOLUTION_HEIGHT, &kheight);
		return 1-kheight;
		}
	if(glIsEnabled(GLAF_SEPARABLE_2D))
		{
		if(glaf_IsConvolutionBorderModesSupported())
			{
			glaf_GetConvolutionParameteriv(GLAF_SEPARABLE_2D, GLAF_CONVOLUTION_BORDER_MODE, &bordermode);
			if(bordermode!=GLAF_REDUCE)
				{
				/*Image keeps the same size after convolution*/
				return 0;
				}
			}
		glaf_GetConvolutionParameteriv(GLAF_SEPARABLE_2D, GLAF_CONVOLUTION_HEIGHT, &kheight);
		return 1-kheight;
		}
	/*Convolution disabled*/
	return 0;
	}
	
	
