/* 
 * 
 * Copyright (c) 2001 Csar Blecua Udas    All Rights Reserved.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * CESAR BLECUA UDIAS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF
 * OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 * 
 */


/* Headers */

#ifdef _WIN32
#include <windows.h>
#endif

#include "glincludes.h"
#include "glaf.h"

/* matrix mode */
static GLint		cp_matrixmode;

/* pixel transfer */
static GLboolean	cp_map_color;
static GLboolean	cp_map_stencil;
static GLint		cp_index_shift;
static GLint		cp_index_offset;
static GLfloat		cp_red_scale;
static GLfloat		cp_green_scale;
static GLfloat		cp_blue_scale;
static GLfloat		cp_alpha_scale;
static GLfloat		cp_depth_scale;
static GLfloat		cp_red_bias;
static GLfloat		cp_green_bias;
static GLfloat		cp_blue_bias;
static GLfloat		cp_alpha_bias;
static GLfloat		cp_depth_bias;

/* color matrix */
static GLfloat		cp_colormatrix[16];
static GLfloat		cp_colormatrix_red_scale;
static GLfloat		cp_colormatrix_green_scale;
static GLfloat		cp_colormatrix_blue_scale;
static GLfloat		cp_colormatrix_alpha_scale;
static GLfloat		cp_colormatrix_red_bias;
static GLfloat		cp_colormatrix_green_bias;
static GLfloat		cp_colormatrix_blue_bias;
static GLfloat		cp_colormatrix_alpha_bias;

/* color table */
static GLboolean	cp_color_table;
static GLboolean	cp_post_convolution_color_table;
static GLboolean	cp_post_color_matrix_color_table;

/* convolution */
static GLboolean	cp_convolution_1d;
static GLboolean	cp_convolution_2d;
static GLboolean	cp_separable_2d;
static GLfloat		cp_convolution_red_scale;
static GLfloat		cp_convolution_green_scale;
static GLfloat		cp_convolution_blue_scale;
static GLfloat		cp_convolution_alpha_scale;
static GLfloat		cp_convolution_red_bias;
static GLfloat		cp_convolution_green_bias;
static GLfloat		cp_convolution_blue_bias;
static GLfloat		cp_convolution_alpha_bias;

/* histogram */
static GLboolean	cp_histogram;
static GLboolean	cp_minmax;

/* pixel store (unpack) */
static GLboolean	cp_unpack_swap_bytes;
static GLboolean	cp_unpack_lsb_first;
static GLint		cp_unpack_alignment;
static GLint		cp_unpack_row_length;
static GLint		cp_unpack_skip_pixels;
static GLint		cp_unpack_skip_rows;

/* pixel store (pack) */
static GLboolean	cp_pack_swap_bytes;
static GLboolean	cp_pack_lsb_first;
static GLint		cp_pack_alignment;
static GLint		cp_pack_row_length;
static GLint		cp_pack_skip_pixels;
static GLint		cp_pack_skip_rows;


void	glaf_StorePixelReadState(void)
	{
	/* A subset of glPushAttrib(GL_PIXEL_MODE_BIT). Saves a copy of a subset of the OpenGL state into internal variables. Just one 
	   copy can be kept at a time, so another call to glaf_StorePixelReadState() will overwrite the results of the previous call.
	   
	   The state subset supported by glaf_StorePixelReadState() can be reset to defaults with glaf_ResetPixelReadState(), or
	   restored to the value of a previous glaf_StorePixelReadState() call with glaf_RestorePixelReadState()
	   
	   This state subset is the one that affects the values of pixels transfered through glPixelRead() and texture loading
	   operations. This subset is *not* suitable for glDrawPixels, since that function uses a wider state.
	
	   NOTE:
		-This could be done with glPushAttrib(GL_PIXEL_MODE_BIT), but I'm not confident about all implementations honoring the
		 imaging extensions state...
	  
	   	-The stored copy of the state is independent from the ones of glaf_StorePixelUnpackState and glaf_StorePixelPackState,
		 so it's possible to call these functions independently, in any order, and without risk of side effects.
	*/
	
	/*pixel transfer state*/
	glGetBooleanv(GL_MAP_COLOR,		&cp_map_color);
	glGetBooleanv(GL_MAP_STENCIL,		&cp_map_stencil);
	glGetIntegerv(GL_INDEX_SHIFT,		&cp_index_shift);
	glGetIntegerv(GL_INDEX_OFFSET,		&cp_index_offset);
	glGetFloatv(GL_RED_SCALE,		&cp_red_scale);	
	glGetFloatv(GL_GREEN_SCALE,		&cp_green_scale);	
	glGetFloatv(GL_BLUE_SCALE,		&cp_blue_scale);	
	glGetFloatv(GL_ALPHA_SCALE,		&cp_alpha_scale);	
	glGetFloatv(GL_DEPTH_SCALE,		&cp_depth_scale);	
	glGetFloatv(GL_RED_BIAS,		&cp_red_bias);		
	glGetFloatv(GL_GREEN_BIAS,		&cp_green_bias);	
	glGetFloatv(GL_BLUE_BIAS,		&cp_blue_bias);	
	glGetFloatv(GL_ALPHA_BIAS,		&cp_alpha_bias);	
	glGetFloatv(GL_DEPTH_BIAS,		&cp_depth_bias);	
	
	/*color matrix state*/
	if(glaf_IsColorMatrixSupported())
		{
		glGetFloatv(GLAF_COLOR_MATRIX,(GLfloat *)&cp_colormatrix);
		glGetFloatv(GLAF_POST_COLOR_MATRIX_RED_SCALE,	&cp_colormatrix_red_scale);	
		glGetFloatv(GLAF_POST_COLOR_MATRIX_GREEN_SCALE,	&cp_colormatrix_green_scale);	
		glGetFloatv(GLAF_POST_COLOR_MATRIX_BLUE_SCALE,	&cp_colormatrix_blue_scale);	
		glGetFloatv(GLAF_POST_COLOR_MATRIX_ALPHA_SCALE,	&cp_colormatrix_alpha_scale);	
		glGetFloatv(GLAF_POST_COLOR_MATRIX_RED_BIAS,	&cp_colormatrix_red_bias);		
		glGetFloatv(GLAF_POST_COLOR_MATRIX_GREEN_BIAS,	&cp_colormatrix_green_bias);	
		glGetFloatv(GLAF_POST_COLOR_MATRIX_BLUE_BIAS,	&cp_colormatrix_blue_bias);	
		glGetFloatv(GLAF_POST_COLOR_MATRIX_ALPHA_BIAS,	&cp_colormatrix_alpha_bias);	
		}

	/*color table state*/
	if(glaf_IsColorTableSupported())
		{
		cp_color_table=glIsEnabled(GLAF_COLOR_TABLE);
		cp_post_convolution_color_table=glIsEnabled(GLAF_POST_CONVOLUTION_COLOR_TABLE);
		cp_post_color_matrix_color_table=glIsEnabled(GLAF_POST_COLOR_MATRIX_COLOR_TABLE);
		}
	
	/*convolution state*/
	if(glaf_IsConvolutionSupported())
		{
		cp_convolution_1d=glIsEnabled(GLAF_CONVOLUTION_1D);
		cp_convolution_2d=glIsEnabled(GLAF_CONVOLUTION_2D);
		cp_separable_2d=glIsEnabled(GLAF_SEPARABLE_2D);
		glGetFloatv(GLAF_POST_CONVOLUTION_RED_SCALE,	&cp_convolution_red_scale);	
		glGetFloatv(GLAF_POST_CONVOLUTION_GREEN_SCALE,	&cp_convolution_green_scale);	
		glGetFloatv(GLAF_POST_CONVOLUTION_BLUE_SCALE,	&cp_convolution_blue_scale);	
		glGetFloatv(GLAF_POST_CONVOLUTION_ALPHA_SCALE,	&cp_convolution_alpha_scale);	
		glGetFloatv(GLAF_POST_CONVOLUTION_RED_BIAS,	&cp_convolution_red_bias);		
		glGetFloatv(GLAF_POST_CONVOLUTION_GREEN_BIAS,	&cp_convolution_green_bias);	
		glGetFloatv(GLAF_POST_CONVOLUTION_BLUE_BIAS,	&cp_convolution_blue_bias);	
		glGetFloatv(GLAF_POST_CONVOLUTION_ALPHA_BIAS,	&cp_convolution_alpha_bias);	
		}		
	
	/*histogram*/
	if(glaf_IsHistogramSupported())
		{
		cp_histogram=glIsEnabled(GLAF_HISTOGRAM);
		cp_minmax=glIsEnabled(GLAF_MINMAX);
		}
	}
		

void	glaf_ResetPixelReadState(void)
	{
	/* Resets a subset of the OpenGL state to the default OpenGL values.
	
	   This state subset is the one that affects the values of pixels transfered through glPixelRead() and texture loading
	   operations. This subset is not suitable for glDrawPixels, since that function uses a wider state.
	
	   NOTE: Read the glaf_StorePixelReadState() documentation for more details.
	*/
	
	/*pixel transfer*/
	glPixelTransferi(GL_MAP_COLOR,		GL_FALSE);
	glPixelTransferi(GL_MAP_STENCIL,	GL_FALSE);
	glPixelTransferi(GL_INDEX_SHIFT,	0);
	glPixelTransferi(GL_INDEX_OFFSET,	0);
	glPixelTransferf(GL_RED_SCALE,		1.0);	
	glPixelTransferf(GL_GREEN_SCALE,	1.0);	
	glPixelTransferf(GL_BLUE_SCALE,		1.0);	
	glPixelTransferf(GL_ALPHA_SCALE,	1.0);	
	glPixelTransferf(GL_DEPTH_SCALE,	1.0);	
	glPixelTransferf(GL_RED_BIAS,		0.0);		
	glPixelTransferf(GL_GREEN_BIAS,		0.0);	
	glPixelTransferf(GL_BLUE_BIAS,		0.0);	
	glPixelTransferf(GL_ALPHA_BIAS,		0.0);	
	glPixelTransferf(GL_DEPTH_BIAS,		0.0);	
	
	/*color matrix*/
	if(glaf_IsColorMatrixSupported())
		{
		glGetIntegerv(GL_MATRIX_MODE,	&cp_matrixmode);
		glMatrixMode(GLAF_COLOR);
		glLoadIdentity();
		glMatrixMode(cp_matrixmode);
		glPixelTransferf(GLAF_POST_COLOR_MATRIX_RED_SCALE,	1.0);	
		glPixelTransferf(GLAF_POST_COLOR_MATRIX_GREEN_SCALE,	1.0);	
		glPixelTransferf(GLAF_POST_COLOR_MATRIX_BLUE_SCALE,	1.0);	
		glPixelTransferf(GLAF_POST_COLOR_MATRIX_ALPHA_SCALE,	1.0);	
		glPixelTransferf(GLAF_POST_COLOR_MATRIX_RED_BIAS,	0.0);		
		glPixelTransferf(GLAF_POST_COLOR_MATRIX_GREEN_BIAS,	0.0);	
		glPixelTransferf(GLAF_POST_COLOR_MATRIX_BLUE_BIAS,	0.0);	
		glPixelTransferf(GLAF_POST_COLOR_MATRIX_ALPHA_BIAS,	0.0);	
		}
	
	/*color table*/
	if(glaf_IsColorTableSupported())
		{
		glDisable(GLAF_COLOR_TABLE);
		glDisable(GLAF_POST_CONVOLUTION_COLOR_TABLE);
		glDisable(GLAF_POST_COLOR_MATRIX_COLOR_TABLE);
		}
	
	/*convolution*/
	if(glaf_IsConvolutionSupported())
		{
		glDisable(GLAF_CONVOLUTION_1D);
		glDisable(GLAF_CONVOLUTION_2D);
		glDisable(GLAF_SEPARABLE_2D);
		glPixelTransferf(GLAF_POST_CONVOLUTION_RED_SCALE,	1.0);	
		glPixelTransferf(GLAF_POST_CONVOLUTION_GREEN_SCALE,	1.0);	
		glPixelTransferf(GLAF_POST_CONVOLUTION_BLUE_SCALE,	1.0);	
		glPixelTransferf(GLAF_POST_CONVOLUTION_ALPHA_SCALE,	1.0);	
		glPixelTransferf(GLAF_POST_CONVOLUTION_RED_BIAS,	0.0);		
		glPixelTransferf(GLAF_POST_CONVOLUTION_GREEN_BIAS,	0.0);	
		glPixelTransferf(GLAF_POST_CONVOLUTION_BLUE_BIAS,	0.0);	
		glPixelTransferf(GLAF_POST_CONVOLUTION_ALPHA_BIAS,	0.0);	
		}
	
	/*histogram*/
	if(glaf_IsHistogramSupported())
		{
		glDisable(GLAF_HISTOGRAM);
		glDisable(GLAF_MINMAX);
		}
	}
	
		
void	glaf_RestorePixelReadState(void)
	{
	/* Restores a subset of the OpenGL state to a previous state stored with a call to glaf_StorePixelReadState().
	
	   This state subset is the one that affects the values of pixels transfered through glPixelRead() and texture loading
	   operations. This subset is not suitable for glDrawPixels, since that function uses a wider state.
	
	   IMPORTANT: Calling this function before any call to glaf_StorePixelReadState() will probably garbage the OpenGL state,
	   	      and the consequences can be *bad*.
	
	   NOTE: Read the glaf_StorePixelReadState() documentation for more details.
	*/
	
	/*pixel transfer*/
	glPixelTransferi(GL_MAP_COLOR,		cp_map_color);
	glPixelTransferi(GL_MAP_STENCIL,	cp_map_stencil);
	glPixelTransferi(GL_INDEX_SHIFT,	cp_index_shift);
	glPixelTransferi(GL_INDEX_OFFSET,	cp_index_offset);
	glPixelTransferf(GL_RED_SCALE,		cp_red_scale);	
	glPixelTransferf(GL_GREEN_SCALE,	cp_green_scale);	
	glPixelTransferf(GL_BLUE_SCALE,		cp_blue_scale);	
	glPixelTransferf(GL_ALPHA_SCALE,	cp_alpha_scale);	
	glPixelTransferf(GL_DEPTH_SCALE,	cp_depth_scale);	
	glPixelTransferf(GL_RED_BIAS,		cp_red_bias);		
	glPixelTransferf(GL_GREEN_BIAS,		cp_green_bias);	
	glPixelTransferf(GL_BLUE_BIAS,		cp_blue_bias);	
	glPixelTransferf(GL_ALPHA_BIAS,		cp_alpha_bias);	
	glPixelTransferf(GL_DEPTH_BIAS,		cp_depth_bias);	
	
	/*color matrix*/
	if(glaf_IsColorMatrixSupported())
		{
		glGetIntegerv(GL_MATRIX_MODE,	&cp_matrixmode);
		glMatrixMode(GLAF_COLOR);
		if((cp_colormatrix[ 0]==1.0F)&&(cp_colormatrix[ 1]==0.0F)&&(cp_colormatrix[ 2]==0.0F)&&(cp_colormatrix[ 3]==0.0F)&&
		   (cp_colormatrix[ 4]==0.0F)&&(cp_colormatrix[ 5]==1.0F)&&(cp_colormatrix[ 6]==0.0F)&&(cp_colormatrix[ 7]==0.0F)&&
		   (cp_colormatrix[ 8]==0.0F)&&(cp_colormatrix[ 9]==0.0F)&&(cp_colormatrix[10]==1.0F)&&(cp_colormatrix[11]==0.0F)&&
		   (cp_colormatrix[12]==0.0F)&&(cp_colormatrix[13]==0.0F)&&(cp_colormatrix[14]==0.0F)&&(cp_colormatrix[15]==1.0F))
		   	{
			/*Let's not play with color matrix -> Put it as easy as possible to the implementation*/
			glLoadIdentity();
			}
		else
			{
			/*Arbitrary matrix*/
			glLoadMatrixf((GLfloat *)&cp_colormatrix);
			}
		glMatrixMode(cp_matrixmode);
		glPixelTransferf(GLAF_POST_COLOR_MATRIX_RED_SCALE,	cp_colormatrix_red_scale);	
		glPixelTransferf(GLAF_POST_COLOR_MATRIX_GREEN_SCALE,	cp_colormatrix_green_scale);	
		glPixelTransferf(GLAF_POST_COLOR_MATRIX_BLUE_SCALE,	cp_colormatrix_blue_scale);	
		glPixelTransferf(GLAF_POST_COLOR_MATRIX_ALPHA_SCALE,	cp_colormatrix_alpha_scale);	
		glPixelTransferf(GLAF_POST_COLOR_MATRIX_RED_BIAS,	cp_colormatrix_red_bias);		
		glPixelTransferf(GLAF_POST_COLOR_MATRIX_GREEN_BIAS,	cp_colormatrix_green_bias);	
		glPixelTransferf(GLAF_POST_COLOR_MATRIX_BLUE_BIAS,	cp_colormatrix_blue_bias);	
		glPixelTransferf(GLAF_POST_COLOR_MATRIX_ALPHA_BIAS,	cp_colormatrix_alpha_bias);	
		}

	/*color table*/
	if(glaf_IsColorTableSupported())
		{
		if(cp_color_table)
			{
			glEnable(GLAF_COLOR_TABLE);
			}
		else
			{
			glDisable(GLAF_COLOR_TABLE);
			}
		if(cp_post_convolution_color_table)
			{
			glEnable(GLAF_POST_CONVOLUTION_COLOR_TABLE);
			}
		else
			{
			glDisable(GLAF_POST_CONVOLUTION_COLOR_TABLE);
			}
		if(cp_post_color_matrix_color_table)
			{
			glEnable(GLAF_POST_COLOR_MATRIX_COLOR_TABLE);
			}
		else
			{
			glDisable(GLAF_POST_COLOR_MATRIX_COLOR_TABLE);
			}
		}
	
	/*convolution*/
	if(glaf_IsConvolutionSupported())
		{
		if(cp_convolution_1d)
			{
			glEnable(GLAF_CONVOLUTION_1D);
			}
		else
			{
			glDisable(GLAF_CONVOLUTION_1D);
			}
		if(cp_convolution_2d)
			{
			glEnable(GLAF_CONVOLUTION_2D);
			}
		else
			{
			glDisable(GLAF_CONVOLUTION_2D);
			}
		if(cp_separable_2d)
			{
			glEnable(GLAF_SEPARABLE_2D);
			}
		else
			{
			glDisable(GLAF_SEPARABLE_2D);
			}
		glPixelTransferf(GLAF_POST_CONVOLUTION_RED_SCALE,	cp_convolution_red_scale);	
		glPixelTransferf(GLAF_POST_CONVOLUTION_GREEN_SCALE,	cp_convolution_green_scale);	
		glPixelTransferf(GLAF_POST_CONVOLUTION_BLUE_SCALE,	cp_convolution_blue_scale);	
		glPixelTransferf(GLAF_POST_CONVOLUTION_ALPHA_SCALE,	cp_convolution_alpha_scale);	
		glPixelTransferf(GLAF_POST_CONVOLUTION_RED_BIAS,	cp_convolution_red_bias);		
		glPixelTransferf(GLAF_POST_CONVOLUTION_GREEN_BIAS,	cp_convolution_green_bias);	
		glPixelTransferf(GLAF_POST_CONVOLUTION_BLUE_BIAS,	cp_convolution_blue_bias);	
		glPixelTransferf(GLAF_POST_CONVOLUTION_ALPHA_BIAS,	cp_convolution_alpha_bias);	
		}
	
	/*histogram*/
	if(glaf_IsHistogramSupported())
		{
		if(cp_histogram)
			{
			glEnable(GLAF_HISTOGRAM);
			}
		else
			{
			glDisable(GLAF_HISTOGRAM);
			}
		if(cp_minmax)
			{
			glEnable(GLAF_MINMAX);
			}
		else
			{
			glDisable(GLAF_MINMAX);
			}
		}
	}	
	
	
void	glaf_StorePixelUnpackState(void)
	{
	/* Saves a copy of a subset of the OpenGL state into internal variables. Just one copy can be kept at a time, so another
	   call to glaf_StorePixelUnpackState() will overwrite the results of the previous call.
	   
	   This state subset is the part of glPixelStore for UNPACK operations (3D texture params are NOT included).
	   
	   The state subset supported by glaf_StorePixelUnpackState() can be reset to defaults with glaf_ResetPixelUnpackState(), or
	   restored to the value of a previous glaf_StorePixelUnpackState() call with glaf_RestorePixelUnpackState()
	   
	   NOTES:
	   	-The stored copy of the state is independent from the ones of glaf_StorePixelReadState and glaf_StorePixelPackState,
		 so it's possible to call these functions independently, in any order, and without risk of interference.
		 
		-Since GLAF doesn't support 3D textures yet, the state for the two PixelStore enumerants for 
		 3D textures (IMAGE_HEIGHT and SKIP_IMAGES) is *not* saved by this function.
	*/
	
	glGetBooleanv(GL_UNPACK_SWAP_BYTES, 	&cp_unpack_swap_bytes);
	glGetBooleanv(GL_UNPACK_LSB_FIRST,  	&cp_unpack_lsb_first);
	glGetIntegerv(GL_UNPACK_ALIGNMENT,  	&cp_unpack_alignment);
	glGetIntegerv(GL_UNPACK_ROW_LENGTH, 	&cp_unpack_row_length);
	glGetIntegerv(GL_UNPACK_SKIP_PIXELS,	&cp_unpack_skip_pixels);
	glGetIntegerv(GL_UNPACK_SKIP_ROWS,  	&cp_unpack_skip_rows);
	}
	
	
void	glaf_ResetPixelUnpackState(void)
	{
	/* Resets a subset of the OpenGL state to the default OpenGL values.
	
	   This state subset is the part of glPixelStore for UNPACK operations (3D texture params are NOT included).
	   
	   NOTES:
	   	-Don't forget that the default OpenGL value for alignment is 4.
		
		-Read the glaf_StorePixelUnpackState() documentation for more details.
	*/
	
	glPixelStorei(GL_UNPACK_SWAP_BYTES, 	GL_FALSE);
	glPixelStorei(GL_UNPACK_LSB_FIRST,  	GL_FALSE);
	glPixelStorei(GL_UNPACK_ALIGNMENT,  	4);
	glPixelStorei(GL_UNPACK_ROW_LENGTH, 	0);
	glPixelStorei(GL_UNPACK_SKIP_PIXELS,	0);
	glPixelStorei(GL_UNPACK_SKIP_ROWS,  	0);
	}
	
	
void	glaf_RestorePixelUnpackState(void)
	{
	/* Restores a subset of the OpenGL state to a previous state stored with a call to glaf_StorePixelUnpackState().
	
	   This state subset is the part of glPixelStore for UNPACK operations (3D texture params are NOT included).
	
	   IMPORTANT: Calling this function before any call to glaf_StorePixelUnpackState() will probably garbage the OpenGL state,
	   	      and the consequences can be *bad*.
	
	   NOTE: Read the glaf_StorePixelUnpackState() documentation for more details.
	*/
	
	glPixelStorei(GL_UNPACK_SWAP_BYTES, 	cp_unpack_swap_bytes);
	glPixelStorei(GL_UNPACK_LSB_FIRST,  	cp_unpack_lsb_first);
	glPixelStorei(GL_UNPACK_ALIGNMENT,  	cp_unpack_alignment);
	glPixelStorei(GL_UNPACK_ROW_LENGTH, 	cp_unpack_row_length);
	glPixelStorei(GL_UNPACK_SKIP_PIXELS,	cp_unpack_skip_pixels);
	glPixelStorei(GL_UNPACK_SKIP_ROWS,  	cp_unpack_skip_rows);
	}
	
	
void	glaf_StorePixelPackState(void)
	{
	/* Saves a copy of a subset of the OpenGL state into internal variables. Just one copy can be kept at a time, so another
	   call to glaf_StorePixelPackState() will overwrite the results of the previous call.
	   
	   This state subset is the part of glPixelStore for PACK operations (3D texture params are NOT included).
	   
	   The state subset supported by glaf_StorePixelPackState() can be reset to defaults with glaf_ResetPixelPackState(), or
	   restored to the value of a previous glaf_StorePixelPackState() call with glaf_RestorePixelPackState()
	   
	   NOTES:
	   	-The stored copy of the state is independent from the ones of glaf_StorePixelReadState and glaf_StorePixelUnpackState,
		 so it's possible to call these functions independently, in any order, and without risk of interference.
		 
		-Since GLAF doesn't support 3D textures yet, the state of the two PixelStore enumerants for 
		 3D textures (IMAGE_HEIGHT and SKIP_IMAGES) is *not* saved by this function.
	*/
	
	glGetBooleanv(GL_PACK_SWAP_BYTES,   	&cp_pack_swap_bytes);
	glGetBooleanv(GL_PACK_LSB_FIRST,    	&cp_pack_lsb_first);
	glGetIntegerv(GL_PACK_ALIGNMENT,    	&cp_pack_alignment);
	glGetIntegerv(GL_PACK_ROW_LENGTH,   	&cp_pack_row_length);
	glGetIntegerv(GL_PACK_SKIP_PIXELS,  	&cp_pack_skip_pixels);
	glGetIntegerv(GL_PACK_SKIP_ROWS,    	&cp_pack_skip_rows);
	}
	
	
void	glaf_ResetPixelPackState(void)
	{
	/* Resets a subset of the OpenGL state to the default OpenGL values.
	
	   This state subset is the part of glPixelStore for PACK operations (3D texture params are NOT included).
	   
	   NOTES:
	   	-Don't forget that the default OpenGL value for alignment is 4.
		
		-Read the glaf_StorePixelPackState() documentation for more details.
	*/
	
	glPixelStorei(GL_PACK_SWAP_BYTES,   	GL_FALSE);
	glPixelStorei(GL_PACK_LSB_FIRST,    	GL_FALSE);
	glPixelStorei(GL_PACK_ALIGNMENT,    	4);
	glPixelStorei(GL_PACK_ROW_LENGTH,   	0);
	glPixelStorei(GL_PACK_SKIP_PIXELS,  	0);
	glPixelStorei(GL_PACK_SKIP_ROWS,    	0);
	}
		
	
void	glaf_RestorePixelPackState(void)
	{
	/* Restores a subset of the OpenGL state to a previous state stored with a call to glaf_StorePixelPackState().
	
	   This state subset is the part of glPixelStore for PACK operations (3D texture params are NOT included).
	
	   IMPORTANT: Calling this function before any call to glaf_StorePixelPackState() will probably garbage the OpenGL state,
	   	      and the consequences can be *bad*.
	
	   NOTE: Read the glaf_StorePixelPackState() documentation for more details.
	*/
	
	glPixelStorei(GL_PACK_SWAP_BYTES,   	cp_pack_swap_bytes);
	glPixelStorei(GL_PACK_LSB_FIRST,    	cp_pack_lsb_first);
	glPixelStorei(GL_PACK_ALIGNMENT,    	cp_pack_alignment);
	glPixelStorei(GL_PACK_ROW_LENGTH,   	cp_pack_row_length);
	glPixelStorei(GL_PACK_SKIP_PIXELS,  	cp_pack_skip_pixels);
	glPixelStorei(GL_PACK_SKIP_ROWS,    	cp_pack_skip_rows);
	}	
	
